"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var uniq = require("lodash.uniq");
var sum = require("lodash.sum");

function isParamIndices(paramIndices, numberOfParams) {
  if (paramIndices && (typeof paramIndices === "undefined" ? "undefined" : _typeof(paramIndices)) === "object" && !Array.isArray(paramIndices)) {
    paramIndices = Object.keys(paramIndices).map(function (name) {
      return Object.assign({ name: name }, paramIndices[name]);
    });
  }

  if (!Array.isArray(paramIndices)) {
    return false;
  }

  if (!paramIndices.every(isValidParamIndex)) {
    return false;
  }

  var paramNames = paramIndices.map(function (paramIndex) {
    return paramIndex.name;
  });

  if (paramNames.length !== uniq(paramNames).length) {
    return false;
  }

  var slots = paramIndices.reduce(fillSlot, []);

  return slots.length === numberOfParams && sum(slots) === numberOfParams;
}

function isValidParamIndex(paramIndex) {
  if (!isParamName(paramIndex.name)) {
    return false;
  }
  if (!isPositiveInteger(paramIndex.index)) {
    return false;
  }
  if (!isPositiveInteger(paramIndex.length)) {
    return false;
  }
  return true;
}

function fillSlot(slots, _ref) {
  var index = _ref.index,
      length = _ref.length;

  for (var i = index; i < index + length; i++) {
    slots[i] = slots[i] ? slots[i] + 1 : 1;
  }
  return slots;
}

function isParamName(value) {
  return typeof value === "string" && /^[_a-zA-Z]\w*$/.test(value);
}

function isPositiveInteger(value) {
  return Math.abs(value | 0) === value;
}

module.exports = isParamIndices;