"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var uniq = require("lodash.uniq");

function isVariants(variants, numberOfParams) {
  if (variants && (typeof variants === "undefined" ? "undefined" : _typeof(variants)) === "object" && !Array.isArray(variants)) {
    variants = Object.keys(variants).map(function (name) {
      return { name: name, values: variants[name] };
    });
  }

  if (!Array.isArray(variants)) {
    return false;
  }

  for (var i = 0; i < variants.length; i++) {
    if (!isValidVariant(variants[i], numberOfParams)) {
      return false;
    }
  }

  var variantNames = variants.map(function (variant) {
    return variant.name;
  });

  if (variantNames.length !== uniq(variantNames).length) {
    return false;
  }

  return true;
}

function isValidVariant(variant, numberOfParams) {
  if (!isVariantName(variant.name)) {
    return false;
  }
  if (!Array.isArray(variant.values)) {
    return false;
  }
  if (variant.values.length !== numberOfParams) {
    return false;
  }
  return variant.values.every(isValidVariantValue);
}

function isVariantName(value) {
  return typeof value === "string" && /^[_a-zA-Z]\w*$/.test(value);
}

function isValidVariantValue(value) {
  return !Number.isNaN(+value);
}

module.exports = isVariants;